#!/usr/bin/ruby
# encoding: utf-8

class BackupScreen

	def initialize (layers, fixed)
		@layers = layers
		@fixed = fixed
		@target_combo = nil
		@targetnet_combo = nil 
		@partitions = Array.new 
		@flat_smb_shares = Array.new 
		@addinput = nil
		@userinput = nil
		@passinput = nil
		@targetnetradio = nil
		@protoradio = Array.new
		lang = ENV['LANGUAGE'][0..1]
		lang = ENV['LANG'][0..1] if lang.nil?
		lang = "en" if lang.nil?
		@tl = MfsTranslator.new(lang, "BackupScreen.xml") 
		@start_label = @tl.get_translation("start_label") 
		@davservices = [ 
			"T-Online Mediacenter", # DAV
			"GMX Mediacenter",  # DAV
			"1&1 SmartDrive",  # DAV
			"Strato HiDrive", # DAV
			"WEB.de", # DAV
			"MD Cloud" # DAV
		]
		@wwwservices = [
			"Apple iCloud", # Website 
			"Amazon Cloud Drive", # Website 
			"Google Drive", # Website
			"Microsoft OneDrive", # Website
			"DropBox" # Website
		]
		@davurls = [ 
			"https://webdav.mediencenter.t-online.de/", 
			"https://webdav.mc.gmx.net/", 
			"https://webdav.office.1und1.de/", 
			"https://webdav.hidrive.strato.com/", 
			"https://webdav.smartdrive.web.de/",
			"https://cloud.md.de/webdav"
		]
		@wwwurls = [
			"https://www.icloud.com/",
			"https://www.amazon.de/clouddrive/home/",
			"https://drive.google.com/", 
			"https://onedrive.live.com/", 
			"https://www.dropbox.com/de"
		] 
		create_first_layer 
		create_credentials_layer
		create_ready_layer
	end
	attr_reader :start_label , :targetnetradio
	
	def create_first_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		cancel = Gtk::Button.new(Gtk::Stock::CANCEL) 
		ok = Gtk::Button.new(Gtk::Stock::APPLY)
		# set = Gtk::Button.new(Gtk::Stock::PREFERENCES) 
		fx.put(bgimg, 0, 0)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("backup_start") )
		fx.put(infolabel, 250, 100)
		# Target local
		target_label = Gtk::RadioButton.new(@tl.get_translation("target-is-local") )
		@target_combo = Gtk::ComboBox.new
		@target_combo.width_request = 730
		@target_combo.height_request = 32
		fx.put(@target_combo, 250, 280)
		fx.put(target_label, 250, 248)
		# Target net
		targetnet_label = Gtk::RadioButton.new(target_label, @tl.get_translation("target-is-network") )
		@targetnet_combo = Gtk::ComboBox.new
		@targetnet_combo.width_request = 730
		@targetnet_combo.height_request = 32
		@targetnet_combo.sensitive = false 
		fx.put(@targetnet_combo, 250, 360)
		fx.put(targetnet_label, 250, 328)
		[ ok, cancel ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		cancel.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			@layers["start"].show_all 
		}
		ok.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			system("su surfer -c 'Thunar --daemon' &")
			if target_label.active?
				mount_local_drives(@partitions[@target_combo.active]) 
				system("su surfer -c 'Thunar /media/backup' &")
				system("su surfer -c 'Thunar /media/disk' &")
				@layers["backup_ready"].show_all 
			else
				if @targetnet_combo.active == @flat_smb_shares.size + @davservices.size + @wwwservices.size
					$stderr.puts "-> Selected last element: Manually specify credentials" 
					@addinput.text = ""
					@protoradio[0].active = true
					@layers["backup_credentials"].show_all 
				elsif @targetnet_combo.active < @flat_smb_shares.size + @davservices.size + @wwwservices.size &&
					@targetnet_combo.active >= @flat_smb_shares.size + @davservices.size 
					$stderr.puts "-> Selected browser openable service" 
					url = @wwwurls[@targetnet_combo.active - @flat_smb_shares.size - @davservices.size ]
					mount_local_drives
					system("su surfer -c 'firefox #{url}' &")
					system("su surfer -c 'Thunar /media/disk' &")
					@layers["backup_ready"].show_all 
				elsif @targetnet_combo.active < @flat_smb_shares.size + @davservices.size &&
					@targetnet_combo.active >= @flat_smb_shares.size
					$stderr.puts "-> Selected DAV service"
					@protoradio[1].active = true
					@addinput.text = @davurls[@targetnet_combo.active - @flat_smb_shares.size ]
					@layers["backup_credentials"].show_all 
				else
					$stderr.puts "-> Selected CIFS share in local network service #{@flat_smb_shares[@targetnet_combo.active]}"
					@protoradio[0].active = true
					@addinput.text = "//#{@flat_smb_shares[@targetnet_combo.active][0]}/#{@flat_smb_shares[@targetnet_combo.active][1]}" 
					@layers["backup_credentials"].show_all 
				end
			end
		}
		target_label.signal_connect("clicked") {
			if target_label.active?
				@target_combo.sensitive = true
				@targetnet_combo.sensitive = false
			else
				@target_combo.sensitive = false
				@targetnet_combo.sensitive = true
			end
		}
		targetnet_label.signal_connect("clicked") {
			if target_label.active?
				@target_combo.sensitive = true
				@targetnet_combo.sensitive = false
			else
				@target_combo.sensitive = false
				@targetnet_combo.sensitive = true
			end
		}
		@targetnetradio = targetnet_label
		targetnet_label.sensitive = false
		fx.put(ok, OK_X, OK_Y)
		fx.put(cancel, CANCEL_X, CANCEL_Y)
		# fx.put(set, SETTINGS_X, SETTINGS_Y)
		@fixed.put(fx, 0, 0)
		@layers["backup_start"] = fx
	end
	
	def create_ready_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		cancel = Gtk::Button.new(Gtk::Stock::CANCEL) 
		[ cancel ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		cancel.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			system("killall -9 Thunar")
			umount_all_drives
			@layers["start"].show_all 
		}
		# ok = Gtk::Button.new(Gtk::Stock::APPLY)
		# set = Gtk::Button.new(Gtk::Stock::PREFERENCES) 
		fx.put(bgimg, 0, 0)
		fx.put(cancel, OK_X, OK_Y)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("backup_ready") )
		fx.put(infolabel, 250, 100)
		@fixed.put(fx, 0, 0)
		@layers["backup_ready"] = fx
	end
	
	def create_credentials_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		cancel = Gtk::Button.new(Gtk::Stock::CANCEL) 
		ok = Gtk::Button.new(Gtk::Stock::APPLY)
		# set = Gtk::Button.new(Gtk::Stock::PREFERENCES) 
		fx.put(bgimg, 0, 0)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("backup-credentials") )
		fx.put(infolabel, 250, 100)
		# Input field address
		addlabel = Gtk::Label.new(@tl.get_translation("server-address")) 
		addlabel.height_request = 32
		fx.put(addlabel, 250, 168)
		# Actual input field 
		@addinput = Gtk::Entry.new
		@addinput.height_request = 32
		@addinput.width_request = 534
		fx.put(@addinput, 458, 168)
		# Username and password
		userlabel = Gtk::Label.new(@tl.get_translation("username"))
		userlabel.height_request = 32
		fx.put(userlabel, 250, 208)
		passlabel = Gtk::Label.new(@tl.get_translation("password"))
		passlabel.height_request = 32
		fx.put(passlabel, 250, 248)
		@userinput = Gtk::Entry.new
		@userinput.height_request = 32
		@userinput.width_request = 534
		fx.put(@userinput, 458, 208)
		@passinput = Gtk::Entry.new
		@passinput.visibility = false 
		@passinput.height_request = 32
		@passinput.width_request = 534
		fx.put(@passinput, 458, 248)
		
		# Label for protocols 
		protolabel = Gtk::Label.new(@tl.get_translation("server-protocol")) 
		protolabel.height_request = 32
		fx.put(protolabel, 250, 288)
		# Radiobuttons for protocol
		@protoradio[0] = Gtk::RadioButton.new(@tl.get_translation("proto-cifs"))
		@protoradio[1] = Gtk::RadioButton.new(@protoradio[0], @tl.get_translation("proto-webdav"))
		@protoradio[2] = Gtk::RadioButton.new(@protoradio[0], @tl.get_translation("proto-https"))
		fx.put(@protoradio[0], 458, 288)
		fx.put(@protoradio[1], 458, 328)
		fx.put(@protoradio[2], 458, 368)
		
		[ ok, cancel ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		cancel.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			@layers["backup_start"].show_all 
		}
		ok.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			if @protoradio[0].active? == true
				mount_cifs(@addinput.text, @userinput.text, @passinput.text) 
				mount_local_drives
				system("su surfer -c 'Thunar /media/backup' &")
				system("su surfer -c 'Thunar /media/disk' &")
			elsif @protoradio[1].active? == true
				mount_davfs(@addinput.text, @userinput.text, @passinput.text) 
				mount_local_drives
				system("su surfer -c 'Thunar /media/backup' &")
				system("su surfer -c 'Thunar /media/disk' &")
			else
				mount_local_drives
				system("su surfer -c 'firefox #{@addinput.text}' &")
				system("su surfer -c 'Thunar /media/disk' &")
			end
			@layers["backup_ready"].show_all
		}
		fx.put(ok, OK_X, OK_Y)
		fx.put(cancel, CANCEL_X, CANCEL_Y)
		@fixed.put(fx, 0, 0)
		@layers["backup_credentials"] = fx
	end
	
	def reread_drivelist
		1000.downto(0) { |n|
			begin
				@target_combo.remove_text(n)
			rescue
			end
		}
		drives = Array.new
		@partitions = Array.new
		Dir.entries("/sys/block").each { |l|
			drives.push(MfsDiskDrive.new(l, false)) if l =~ /[a-z]$/ ||  l =~ /mmcblk[0-9]$/ ||  l =~ /mmcblk[0-9][0-9]$/
		}
		drives.reverse.each { |d|
			d.partitions.each  { |p|
				label = "#{d.vendor} #{d.model}"
				if d.usb == true
					label = label + " (USB) "
				else
					label = label + " (SATA/eSATA/IDE) "
				end
				label = label + "#{p.device} #{p.human_size}"
				label = @tl.get_translation("backup_medium") + " " + label if p.label.to_s == "USBDATA"
				pmnt = p.mount_point
				if ( p.fs =~ /vfat/ || p.fs =~ /ntfs/ || p.fs =~ /ext/ || p.fs =~ /btrfs/ ) && pmnt.nil?
					label = label + " #{p.fs}"
					@partitions.push p
					@target_combo.append_text(label)
				end
			}
		}
		@target_combo.active = 0
	end 
	
	def reread_netlist
		@flat_smb_shares = Array.new 
		1000.downto(0) { |n|
			begin
				@targetnet_combo.remove_text(n)
			rescue
			end
		}
		smbshares, hostdescs, workgroups = get_smb_servers
		smbshares.each { |n,s|
			s.each { |m|
				@targetnet_combo.append_text("//#{n}/#{m[0]} (Windows share)")
				@flat_smb_shares.push( [n, m[0], m[1] ] ) 
			}
		}
		(@davservices + @wwwservices).each { |n|
			@targetnet_combo.append_text(n) 
		}
		@targetnet_combo.append_text(@tl.get_translation("manually_enter_credentials"))
		@targetnet_combo.active = 0
	end
	
	def mount_local_drives(target=nil) 
		@partitions.each { |p|
			if p == target 
				FileUtils.mkdir_p("/media/backup")
				p.mount("rw", "/media/backup", 1000, 1000)
			else
				p.mount("ro", nil, 1000, 1000)
				begin
					p.vss_mount_all if p.vss? 
				rescue
				end
			end
		}
	end
	
	def umount_all_drives
		@partitions.each { |p|
			begin
				p.vss_umount_all if p.vss? 
			rescue
			end
			p.force_umount unless p.umount 
		}
		system("umount /media/backup") 
	end
	
	def get_smb_servers(browse_with_pass=false, default_user=nil, default_pass=nil)
		workgroups = Hash.new
		hostdescs = Hash.new
		smbshares = Hash.new
		tree_command = "smbtree -N"
		tree_command = "smbtree -U '" + default_user.to_s + "%" + default_pass.to_s + "'" if browse_with_pass == true
		IO.popen(tree_command) { |io|
			wgroup = nil
			hostname = nil
			while io.gets
				unless $_.strip =~ /\\\\/
					wgroup = $_.strip
				end
				if $_ =~ /^\t\\\\/
					namedesc = $_.strip.split(/\t+/)
					hostname = namedesc[0].strip.gsub('\\', '')
					begin
						hostdescs[hostname] = namedesc[1].strip
					rescue
						hostdescs[hostname] = ""
					end
					workgroups[hostname] = wgroup
					smbshares[hostname] = Array.new
				elsif $_ =~ /^\t\t\\\\/
					sharedesc = $_.strip.split(/\t+/)
					begin
						sharedet = sharedesc[1].strip
					rescue
						sharedet = ""
					end
					smbshares[hostname].push( [ sharedesc[0].strip.gsub(hostname, '').gsub('\\', ''), sharedet ] ) unless
						sharedesc[0].strip.gsub(hostname, '').gsub('\\', '') == 'IPC$' ||
						sharedesc[0].strip.gsub(hostname, '').gsub('\\', '') == 'C$' ||
						sharedesc[0].strip.gsub(hostname, '').gsub('\\', '') == 'ADMIN$'
				end
			end
		}
		return smbshares, hostdescs, workgroups
	end
	
	def mount_cifs(share, user, pass)
		cleaned_share = share.strip
		FileUtils.mkdir_p("/media/backup")
		mount_command = ""
		if (user.strip == "") 
			mount_command = "mount.cifs -o username=guest,password=guest,uid=1000,gid=1000,rw #{cleaned_share} /media/backup"
		else
			mount_command = "mount.cifs -o username=#{user.strip},password=#{pass.strip},uid=1000,gid=1000,rw #{cleaned_share} /media/backup"
		end
		unless system(mount_command)
			cleaned_share =~ /\/\/(.*?)\/(.*?)$/ 
			server = $1 
			path = $2 
			ip = ` nmblookup #{server}`.split("\n")[1].split[0] 
			cleaned_share = "//#{ip}/#{path}"
			if (user.strip == "") 
				mount_command = "mount.cifs -o username=guest,password=guest,uid=1000,gid=1000,rw #{cleaned_share} /media/backup"
			else
				mount_command = "mount.cifs -o username=#{user.strip},password=#{pass.strip},uid=1000,gid=1000,rw #{cleaned_share} /media/backup"
			end
			system(mount_command)
		end
	end	
	
	def mount_davfs(share, user, pass)
		FileUtils.mkdir_p("/media/backup")
		system("echo -e '#{user}\n#{pass}\ny' | LANG=en_GB.UTF-8 LANGUAGE=en mount.davfs -o rw,uid=1000,gid=1000,dir_mode=755,file_mode=644 \"#{share}\" /media/backup")
	end
end
