#!/usr/bin/ruby
# encoding: utf-8

class SambaScreen

	def initialize(layers, fixed)
		@layers = layers
		@fixed = fixed
		@source_combo = nil
		@target_combo = nil 
		@vte = nil
		lang = ENV['LANGUAGE'][0..1]
		lang = ENV['LANG'][0..1] if lang.nil?
		lang = "en" if lang.nil?
		@tl = MfsTranslator.new(lang, "SambaScreen.xml") 
		@start_label = @tl.get_translation("start_label") 
		create_first_layer 
		create_ready_layer
		# create_progress_layer
		# create_vte_layer
		# create_settings_layer 
	end
	attr_reader :start_label
	
	def create_first_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		cancel = Gtk::Button.new(Gtk::Stock::CANCEL) 
		ok = Gtk::Button.new(Gtk::Stock::APPLY)
		fx.put(bgimg, 0, 0)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("samba_start")) 
		fx.put(infolabel, 250, 100)
		[ ok, cancel ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		cancel.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			@layers["start"].show_all 
		}
		ok.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			#while (Gtk.events_pending?)
			#	Gtk.main_iteration
			#end
			start_samba 
			@layers["samba_ready"].show_all 
		}
		fx.put(ok, OK_X, OK_Y)
		fx.put(cancel, CANCEL_X, CANCEL_Y)
		@fixed.put(fx, 0, 0)
		@layers["samba_start"] = fx
	end
	
	def create_ready_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		cancel = Gtk::Button.new(Gtk::Stock::CANCEL) 
		fx.put(bgimg, 0, 0)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("samba_ready")) 
		fx.put(infolabel, 250, 100)
		[ cancel ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		cancel.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			#while (Gtk.events_pending?)
			#	Gtk.main_iteration
			#end
			cleanup 
			@layers["start"].show_all 
		}
		fx.put(cancel, OK_X, OK_Y)
		@fixed.put(fx, 0, 0)
		@layers["samba_ready"] = fx
	end

	def start_samba
		mount_local_drives
		system("smbd -D")
		system("nmbd -D")
	end
	
	def cleanup
		umount_all_drives 
		system("killall -9 smbd") 
		system("killall -9 nmbd") 
	end
	
	def mount_local_drives(target=nil) 
		drives = Array.new
		Dir.entries("/sys/block").each { |l|
			drives.push(MfsDiskDrive.new(l, false)) if l =~ /[a-z]$/ ||  l =~ /mmcblk[0-9]$/ ||  l =~ /mmcblk[0-9][0-9]$/
		}
		drives.each { |d|
			d.partitions.each  { |p|
				if p == target 
					FileUtils.mkdir_p("/media/backup")
					p.mount("rw", "/media/backup", 1000, 1000)
				else
					p.mount("ro", nil, 1000, 1000)
					begin
						p.vss_mount_all if p.vss? 
					rescue
					end
				end
			}
		}
	end
	
	def umount_all_drives
		drives = Array.new
		Dir.entries("/sys/block").each { |l|
			drives.push(MfsDiskDrive.new(l, false)) if l =~ /[a-z]$/ ||  l =~ /mmcblk[0-9]$/ ||  l =~ /mmcblk[0-9][0-9]$/
		}
		drives.each { |d|
			d.partitions.each  { |p|
				begin
					p.vss_umount_all if p.vss? 
				rescue
				end
				p.force_umount unless p.umount 
			}
		}
	end
	
end
