#!/usr/bin/ruby
# encoding: utf-8

class ShredderScreen

	def initialize(layers, fixed)
		@layers = layers
		@fixed = fixed
		@source_combo = nil
		@vte = nil
		@pgbar = nil 
		@ok = nil 
		@done_button = nil 
		@kill_button = nil 
		@shred_running = false 
		@devices = Array.new 
		lang = ENV['LANGUAGE'][0..1]
		lang = ENV['LANG'][0..1] if lang.nil?
		lang = "en" if lang.nil?
		@tl = MfsTranslator.new(lang, "ShredderScreen.xml") 
		@start_label = @tl.get_translation("start_label") 
		create_first_layer 
		create_progress_layer
		# create_vte_layer
		# create_settings_layer 
	end
	attr_reader :start_label
	
	def create_first_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		cancel = Gtk::Button.new(Gtk::Stock::CANCEL) 
		@ok = Gtk::Button.new(Gtk::Stock::APPLY)
		set = Gtk::Button.new(Gtk::Stock::PREFERENCES) 
		fx.put(bgimg, 0, 0)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("shred_start")) 
		fx.put(infolabel, 250, 100)
		# Source
		source_label = Gtk::Label.new(@tl.get_translation("source") )
		@source_combo = Gtk::ComboBox.new
		@source_combo.width_request = 730
		@source_combo.height_request = 32
		fx.put(@source_combo, 250, 240)
		fx.put(source_label, 250, 208)
		[ @ok, cancel, set ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		cancel.signal_connect("clicked") { 
			@shred_running = false 
			@layers.each { |k,v| v.hide_all }
			@layers["start"].show_all 
		}
		@ok.signal_connect("clicked") { 
			if TileHelpers.yes_no_dialog(@tl.get_translation("do_you_really_want"))
				@layers.each { |k,v| v.hide_all }
				@layers["shred_running"].show_all 
				@done_button.sensitive = false
				@kill_button.sensitive = true 
				@shred_running = true 
				errors = run_shred
				@pgbar.fraction = 1.0 
				@pgbar.text = @tl.get_translation("shred_done") 
				@done_button.sensitive =true
				@kill_button.sensitive = false 
				if errors > 0
					TileHelpers.error_dialog(@tl.get_translation("warning_some_errors").gsub("%ERRORS%", errors.to_s)) 
				else 
					TileHelpers.success_dialog(@tl.get_translation("done_without_errors"))
				end
			end
		}
		set.signal_connect("clicked") { 
			@layers.each { |k,v| v.hide_all }
			@layers["shred_settings"].show_all 
		}
		fx.put(@ok, OK_X, OK_Y)
		fx.put(cancel, CANCEL_X, CANCEL_Y)
		# fx.put(set, SETTINGS_X, SETTINGS_Y)
		@fixed.put(fx, 0, 0)
		@layers["shred_start"] = fx
	end
	
	def create_progress_layer
		bgimg = Gtk::Image.new("background.png")
		fx = Gtk::Fixed.new
		@kill_button = Gtk::Button.new(Gtk::Stock::CANCEL) 
		@done_button = Gtk::Button.new(Gtk::Stock::OK)
		@done_button.sensitive = false
		fx.put(bgimg, 0, 0)
		# Label
		infolabel = Gtk::Label.new
		infolabel.wrap = true
		infolabel.width_request = 730
		infolabel.set_markup(@tl.get_translation("shred_running"))
		fx.put(infolabel, 250, 100)
		# Progress Bar
		@pgbar = Gtk::ProgressBar.new
		@pgbar.fraction = 0.3 
		@pgbar.width_request = 730
		@pgbar.height_request = 32
		fx.put(@pgbar, 250, 240)
		@kill_button.signal_connect("clicked") {
			@shred_running = false 
			@layers.each { |k,v| v.hide_all }
			@layers["start"].show_all 
		}
		@done_button.signal_connect("clicked") {
			@layers.each { |k,v| v.hide_all }
			@layers["start"].show_all 
		}
		[ @kill_button, @done_button ].each { |w|
			w.width_request = 128
			w.height_request = 32 
		}
		fx.put(@done_button, OK_X, OK_Y)
		fx.put(@kill_button, CANCEL_X, CANCEL_Y)
		@fixed.put(fx, 0, 0)
		@layers["shred_running"] = fx
	end
	
	def reread_drivelist
		@devices = Array.new 
		1000.downto(0) { |n|
			begin
				@source_combo.remove_text(n)
			rescue
			end
		}
		drives = Array.new
		partitions = Array.new
		Dir.entries("/sys/block").each { |l|
			drives.push(MfsDiskDrive.new(l, false)) if l =~ /[a-z]$/ ||  l =~ /mmcblk[0-9]$/ ||  l =~ /mmcblk[0-9][0-9]$/
		}
		drives.each { |d|
			d.force_umount 
			unless d.mounted == true
				label = "#{d.vendor} #{d.model} #{d.device} - #{d.human_size}"
				if d.usb == true
					label = label + " (USB) "
				else
					label = label + " (SATA/eSATA/IDE) "
				end
				@source_combo.append_text(label)
				@devices.push d 
				d.partitions.each  { |p|
					label = "#{d.vendor} #{d.model} #{p.device} - #{p.human_size}"
					if d.usb == true
						label = label + " (USB) "
					else
						label = label + " (SATA/eSATA/IDE) "
					end
					# label = @tl.get_translation("backup_medium") + " " + label if p.label.to_s == "USBDATA"
					pmnt = p.mount_point
					if pmnt.nil?
						label = label + " #{p.fs.to_s}"
						@devices.push p
						@source_combo.append_text("      " + label)
					end
				}
			end
		}
		if @devices.size < 1
			@source_combo.append_text(@tl.get_translation("no_suitable_device"))
			@source_combo.sensitive = false
			@ok.sensitive = false
		else
			@ok.sensitive = true 
			@source_combo.sensitive = true
		end
		@source_combo.active = 0
	end
	
	def run_shred
		dev = @devices[@source_combo.active]
		blocksize = 4 * 1024 * 1024
		expected_blocks = dev.size / blocksize 
		errors = -1
		$stderr.puts "Shredding: #{dev.device}, size: #{dev.size}"
		0.upto(expected_blocks) { |n|
			@pgbar.fraction = n.to_f / expected_blocks.to_f 
			@pgbar.text = @tl.get_translation("shred_percent_done").gsub("%PERCENT%", (n.to_f / expected_blocks.to_f * 100.0).to_i.to_s)  
			if @shred_running == true 
				ddstr = "dd if=/dev/zero bs=#{blocksize} count=1 of=/dev/#{dev.device} seek=#{n}"
				errors += 1 unless system(ddstr) 
				while (Gtk.events_pending?)
					Gtk.main_iteration
				end
				sleep 0.05 
			end
		}
		system("sync")
		$stderr.puts "Errors: #{errors}"
		return errors 
	end
	
end
