#!/usr/bin/ruby
# encoding: utf-8

require 'gtk2'
require 'vte'
require "rexml/document"
require 'optparse'
require "RescueScreen" 
require "BackupScreen"
require "CloneScreen" 
require "VirusScreen"
require "ShredderScreen"
require "SambaScreen"
require "LicenseScreen"
require "TileHelpers"
require "MfsTranslator"
require "MfsDiskDrive"
require "MfsSinglePartition"
# require "FileUtils"

# Constants for positions:

OK_X = 864
OK_Y = 536
CANCEL_X = 728
CANCEL_Y = 536
SETTINGS_X = 592
SETTINGS_Y = 536

$lastpulse = Time.now.to_f 

lang = ENV['LANGUAGE'][0..1]
lang = ENV['LANG'][0..1] if lang.nil?
lang = "en" if lang.nil?
@tl = MfsTranslator.new(lang, "rescuegui.xml") 

# Identify the correct manual
manualpath = "/lesslinux/cdrom/Manual"
manualpath = "/lesslinux/isoloop/Manual" if File.directory?("/lesslinux/isoloop/Manual")
manualpath = "/lesslinux/cdrom/Manual" if File.directory?("/lesslinux/cdrom/Manual")
@manual = manualpath + "/SECUPERTS_First_Aid_Kit_en-EN.pdf" 
@manual = manualpath + "/SECUPERTS_First_Aid_Kit_de-DE.pdf" if lang == "de"

top_buttons = Array.new

window = Gtk::Window.new
window.set_size_request(1000, 576)
window.border_width = 0
window.window_position = Gtk::Window::POS_CENTER_ALWAYS

window.deletable = false
window.decorated = false
window.allow_grow = false
window.allow_shrink = false
window.title = "RescueGUI"

fixed = Gtk::Fixed.new

# Layers are stored in a Hash:

layers = Hash.new

# First layer for starting page

bgimg = Gtk::Image.new("background.png")
startfixed = Gtk::Fixed.new
layers["start"] = startfixed 
startfixed.put(bgimg, 0, 0)

# Layers provided by component "Rescue"

rescuescreen = RescueScreen.new(layers, fixed) 
rescue_button = Gtk::Button.new()
resc_hbox = Gtk::HBox.new
resc_hbox.pack_start_defaults Gtk::Image.new("icons/datenrettung_gelb.png")
rescue_label = Gtk::Label.new # (rescuescreen.start_label) 
rescue_label.set_markup(rescuescreen.start_label)
rescue_label.wrap = true 
rescue_label.width_request = 230
resc_hbox.pack_start_defaults rescue_label 
rescue_button.add(resc_hbox) 

# rescue_button.image = Gtk::Image.new("icons/datenrettung_gelb.png")
rescue_button.signal_connect("clicked") { 
	layers.each { |k,v| v.hide_all }
	rescuescreen.reread_drivelist 
	layers["rescue_start"].show_all 
}
startfixed.put(rescue_button, 226, 244)

# Layers provided by component "Backup"

backupscreen = BackupScreen.new(layers, fixed) 
backup_button = Gtk::Button.new()
backup_hbox = Gtk::HBox.new
backup_hbox.pack_start_defaults Gtk::Image.new("icons/backup_gelb.png")
backup_label = Gtk::Label.new # (rescuescreen.start_label) 
backup_label.set_markup(backupscreen.start_label)
backup_label.wrap = true 
backup_label.width_request = 230
backup_hbox.pack_start_defaults backup_label 
backup_button.add(backup_hbox) 
backup_button.signal_connect("clicked") { 
	if LicenseScreen.offline_check == true &&  LicenseScreen.online_check == true
		layers.each { |k,v| v.hide_all }
		backupscreen.reread_drivelist 
		backupscreen.reread_netlist
		layers["backup_start"].show_all 
		if LicenseScreen.get_features.include?("Cloudbackup")
			backupscreen.targetnetradio.sensitive = true 
		else
			TileHelpers.error_dialog( @tl.get_translation("cloud_backup_not_licensed") )
		end
	elsif LicenseScreen.offline_check == true
		system("sleep 5 ; wicd-gtk --no-tray &")
		TileHelpers.error_dialog( @tl.get_translation("online_check_failed") )
		layers.each { |k,v| v.hide_all }
		backupscreen.reread_drivelist 
		backupscreen.reread_netlist
		layers["backup_start"].show_all 
		if LicenseScreen.online_check == true
			if LicenseScreen.get_features.include?("Cloudbackup")
				backupscreen.targetnetradio.sensitive = true 
			else
				TileHelpers.error_dialog( @tl.get_translation("cloud_backup_not_licensed") )
			end
		else
			TileHelpers.error_dialog( @tl.get_translation("online_check_failed_continue_anyway") )
		end
	else
		if TileHelpers.yes_no_dialog( @tl.get_translation("cloudbackup_requires_valid_license") ) == true
			layers.each { |k,v| v.hide_all }
			top_buttons.each { |v| v.hide_all }
			layers["license"].show_all 
		else
			layers.each { |k,v| v.hide_all }
			backupscreen.reread_drivelist 
			backupscreen.reread_netlist
			layers["backup_start"].show_all 
		end
	end
}
startfixed.put(backup_button, 226, 94)

# Layers provided by component "Clone"

clonescreen = CloneScreen.new(layers, fixed) 
clone_button = Gtk::Button.new()
clone_hbox = Gtk::HBox.new
clone_hbox.pack_start_defaults  Gtk::Image.new("icons/laufwerkklonen_gelb.png")
clone_label = Gtk::Label.new # (rescuescreen.start_label) 
clone_label.set_markup(clonescreen.start_label)
clone_label.wrap = true 
clone_label.width_request = 230
clone_hbox.pack_start_defaults clone_label 
clone_button.add(clone_hbox) 
clone_button.signal_connect("clicked") { 
	layers.each { |k,v| v.hide_all }
	clonescreen.reread_drivelist 
	layers["clone_start"].show_all 
}
startfixed.put(clone_button, 608, 244)

# Layers provided by component "Virusscan"

virusscreen = VirusScreen.new(layers, fixed) 
virus_button = Gtk::Button.new()
virus_hbox = Gtk::HBox.new
virus_hbox.pack_start_defaults Gtk::Image.new("icons/virusscan-gelb.png") 
virus_label = Gtk::Label.new # (rescuescreen.start_label) 
virus_label.set_markup(virusscreen.start_label)
virus_label.wrap = true 
virus_label.width_request = 230
virus_hbox.pack_start_defaults virus_label 
virus_button.add(virus_hbox) 
virus_button.signal_connect("clicked") { 
	if LicenseScreen.offline_check == true &&  LicenseScreen.online_check == true
		layers.each { |k,v| v.hide_all }
		virusscreen.reread_drivelist
		layers["virus_start"].show_all 
		if LicenseScreen.get_features.include?("HDDCleanup")
			virusscreen.act_radio[1].sensitive = true
			virusscreen.act_radio[2].sensitive = true
			virusscreen.det_radio[0].sensitive = true
			virusscreen.det_radio[1].sensitive = true
		else
			TileHelpers.error_dialog( @tl.get_translation("virus_cleanup_not_licensed") )
		end
	elsif LicenseScreen.offline_check == true
		system("sleep 5 ; wicd-gtk --no-tray &")
		TileHelpers.error_dialog( @tl.get_translation("online_check_failed") )
		layers.each { |k,v| v.hide_all }
		virusscreen.reread_drivelist
		layers["virus_start"].show_all 
		if LicenseScreen.online_check == true
			if LicenseScreen.get_features.include?("HDDCleanup")
				virusscreen.act_radio[1].sensitive = true
				virusscreen.act_radio[2].sensitive = true
				virusscreen.det_radio[0].sensitive = true
				virusscreen.det_radio[1].sensitive = true
			else
				TileHelpers.error_dialog( @tl.get_translation("virus_cleanup_not_licensed") )
			end
		else
			TileHelpers.error_dialog( @tl.get_translation("online_check_failed_continue_anyway") )
		end
	else
		if TileHelpers.yes_no_dialog( @tl.get_translation("virus_requires_valid_license") ) == true
			layers.each { |k,v| v.hide_all }
			top_buttons.each { |v| v.hide_all }
			layers["license"].show_all 
		else
			virusscreen.act_radio[1].sensitive = false
			virusscreen.act_radio[2].sensitive = false
			virusscreen.det_radio[0].sensitive = false
			virusscreen.det_radio[1].sensitive = false
			layers.each { |k,v| v.hide_all }
			virusscreen.reread_drivelist
			layers["virus_start"].show_all 
		end
	end
	# layers.each { |k,v| v.hide_all }
	# virusscreen.reread_drivelist
	# layers["virus_start"].show_all 
}
startfixed.put(virus_button, 608, 94)

# Layers provided by component shredder 

shredscreen = ShredderScreen.new(layers, fixed) 
shred_button = Gtk::Button.new()
shred_hbox = Gtk::HBox.new
shred_hbox.pack_start_defaults Gtk::Image.new("icons/erase.png") 
shred_label = Gtk::Label.new # (rescuescreen.start_label) 
shred_label.set_markup(shredscreen.start_label)
shred_label.wrap = true 
shred_label.width_request = 230
shred_hbox.pack_start_defaults shred_label 
shred_button.add(shred_hbox) 
shred_button.signal_connect("clicked") { 
	layers.each { |k,v| v.hide_all }
	shredscreen.reread_drivelist
	layers["shred_start"].show_all 
}
startfixed.put(shred_button, 608, 394)

# Layers provided by component samba

sambascreen = SambaScreen.new(layers, fixed) 
samba_button = Gtk::Button.new()
samba_hbox = Gtk::HBox.new
samba_hbox.pack_start_defaults Gtk::Image.new("icons/network_gelb.png") 
samba_label = Gtk::Label.new # (rescuescreen.start_label) 
samba_label.set_markup(sambascreen.start_label)
samba_label.wrap = true 
samba_label.width_request = 230
samba_hbox.pack_start_defaults samba_label 
samba_button.add(samba_hbox) 
samba_button.signal_connect("clicked") { 
	if LicenseScreen.offline_check == true &&  LicenseScreen.online_check == true
		if LicenseScreen.get_features.include?("NetworkShares")
			layers.each { |k,v| v.hide_all }
			layers["samba_start"].show_all 
		else
			TileHelpers.error_dialog( @tl.get_translation("samba_not_licensed") )
		end
	elsif LicenseScreen.offline_check == true
		system("sleep 5 ; wicd-gtk --no-tray &")
		TileHelpers.error_dialog( @tl.get_translation("online_check_failed") )
		if LicenseScreen.online_check == true
			if LicenseScreen.get_features.include?("NetworkShares")
				layers.each { |k,v| v.hide_all }
				layers["samba_start"].show_all 
			else
				TileHelpers.error_dialog( @tl.get_translation("samba_not_licensed") )
			end
		end
	else
		if TileHelpers.yes_no_dialog( @tl.get_translation("samba_requires_valid_license") ) == true
			layers.each { |k,v| v.hide_all }
			top_buttons.each { |v| v.hide_all }
			layers["license"].show_all 
		end
	end
}
startfixed.put(samba_button, 226, 394)

# 
fixed.put(startfixed,0,0)

# Icons shown in any mode - must be added latest!

button_count = 0
[ 
	"icons/help.png",
	"icons/browser.png", 
	"icons/network.png",
	"icons/usb.png", 
	"icons/desktopmodus.png",
	"icons/shutdown.png"
].each { |i| 
	ico = Gtk::Button.new # Gtk::EventBox.new
	ico.image = Gtk::Image.new(i)
	fixed.put(ico, 712 + button_count*48, 8)
	top_buttons.push(ico)
	button_count += 1
}
top_commands = [ 
	"evince #{@manual} &", 
	"su surfer -c firefox &",
	"wicd-gtk --no-tray &",
	"usbinstall.sh &",
	nil,
	"shutdown-wrapper.sh"
]
0.upto(top_buttons.size - 1) { |n|
	top_buttons[n].signal_connect('button-release-event') { |x, y|
		if y.button == 1 
			tc = top_commands[n]
			if tc.nil?
				if LicenseScreen.offline_check == true
					if File.exists?("/etc/lesslinux/secuperts/license.xml") 
						if LicenseScreen.get_features.include?("DesktopModus")
							FileUtils.touch("/home/surfer/.ll_requested_desktop")
							FileUtils.chown(1000,1000, "/home/surfer/.ll_requested_desktop")
							Gtk.main_quit
							exit
						else
							TileHelpers.error_dialog( @tl.get_translation("desktop_not_licensed") )
						end
					else
						FileUtils.touch("/home/surfer/.ll_requested_desktop")
						FileUtils.chown(1000,1000, "/home/surfer/.ll_requested_desktop")
						Gtk.main_quit
						exit
					end
				else
					if TileHelpers.yes_no_dialog( @tl.get_translation("desktop_requires_valid_license") )
						layers.each { |k,v| v.hide_all }
						top_buttons.each { |v| v.hide_all }
						layers["license"].show_all 
					end
				end
			else
				system(tc)
			end
		elsif y.button == 3
			layers.each { |k,v| v.hide_all }
			top_buttons.each { |v| v.hide_all }
			layers["license"].show_all 
		end
	}
}
licensescreen = LicenseScreen.new(layers, fixed, top_buttons) 

window.add fixed
window.signal_connect('destroy') { Gtk.main_quit }
window.show_all 
# unshow all layers
layers.each { |k,v| v.hide_all }
top_buttons.each { |v| v.hide_all }
if File.exist?("/var/run/lesslinux/license_accepted") 
	layers["start"].show_all 
	top_buttons.each { |v| v.show_all }
else
	layers["license"].show_all 
	
end		
# layers.each { |k,v| v.hide_all }
# layers["rescue_start"].show_all 

Gtk.main